/* -*- linux-c -*-
 *
 *	Declarations for MOXA C-218 Serial Board Driver
 *
 *	(c) 1997 -- 1999 Martin Mares, <mj@ucw.cz>
 */

/*
 * Types
 */

typedef u8 byte;
typedef u16 word;
typedef unsigned int ulg;

/*
 * This is our internal structure for each serial port's state.
 * 
 * Many fields are paralleled by the structure used by the serial_struct
 * structure.
 *
 * For definitions of the flags field, see tty.h
 */

#define MOXA_MAGIC 0x4d58

struct moxa_port {
	int	magic;
	int	flags;					    /* defined in tty.h */
	int	type;
	struct tty_struct *tty;
	volatile struct moxa_chan_ctrl *ctrl;
	byte	*page_port;	
	byte	*mem_window;
	ulg	read_buffer;
	ulg	write_buffer;
	int	line;					    /* device minor */
	int	count;					    /* # of fd on device */
	int	blocked_open;				    /* # of blocked opens */
	int	close_delay;
	long	session;				    /* Session of opening process */
	long	pgrp;					    /* pgrp of opening process */
	word	event;
	word	modem_ctrl;
	word	modem_ctrl_old;
	word	modem_status;
	word	modem_status_old;
	byte	modem_event_scheduled;
	byte	cmd_in_progress;
	byte	write_in_progress;
	byte	receiver_wakeup;
	byte	modem_events_enabled;
	struct tq_struct	tqueue;
	struct termios		normal_termios;
	struct termios		callout_termios;
	struct wait_queue	*open_wait;
	struct wait_queue	*close_wait;
};

/*
 * Channel control structure.
 */

struct moxa_chan_ctrl {
	word rx_queue_read;		/* 00 */
	word rx_queue_write;		/* 02 */
	word tx_queue_read;		/* 04 */
	word tx_queue_write;		/* 06 */
	word flags1;			/* 08 */
	word status;			/* 0A */
	word flowctrl;			/* 0C */
	word break_cnt;			/* 0E */
	word start_xmit;		/* 10 */
	word unknown[0x17];		/* 12 */
	word command;			/* 40 */
	word par1;			/* 42 */
	word par2;			/* 44 */
	word par3;			/* 46 */
};

#define MOXA_RX_BUF_SIZE 0x8000
#define MOXA_RX_WAKEUP_LEN 0x400
#define MOXA_TX_BUF_SIZE 0x4000
#define MOXA_TX_THRESH 0x3000
#define MOXA_TIMER_PERIOD HZ/8

#define MOXA_FLG1_SEQ_ENABLE	0x01
#define MOXA_FLG1_CNT_ENABLE	0x02
#define MOXA_FLG1_BREAK_EVENT	0x04
#define MOXA_FLG1_MODEM_EVENT	0x08
#define MOXA_FLG1_PORT_OPEN	0x10

#define MOXA_STAT_OVERFLOW	0x01
#define MOXA_STAT_FULL		0x02
#define MOXA_STAT_BREAK		0x04

#define MOXA_FLOW_CTS		0x01
#define MOXA_FLOW_RTS		0x02
#define MOXA_FLOW_TX_XON	0x04
#define MOXA_FLOW_RX_XON	0x08

#define MOXA_MSTAT_CTS		0x01
#define MOXA_MSTAT_DSR		0x02
#define MOXA_MSTAT_RING		0x04
#define MOXA_MSTAT_DCD		0x08

#define MOXA_MCTRL_DTR		0x01
#define MOXA_MCTRL_RTS		0x02

#define MOXA_SWEVENT_TBE	0x01
#define MOXA_SWEVENT_MODEM	0x02
#define MOXA_SWEVENT_READ	0x04
#define MOXA_SWEVENT_BREAK	0x08

/*
 * Card event control structure.
 */

struct moxa_event_ctrl {
word event_table_offset;	/* 00 */
word tmp_event_table_offset;	/* 02 */
word int_flag;			/* 04 */
};

#define MOXA_EVENT_RXBUF_FULL 0x01
#define MOXA_EVENT_SEQUENCE 0x02
#define MOXA_EVENT_COUNT 0x04
#define MOXA_EVENT_BREAK 0x08
#define MOXA_EVENT_MODEM 0x10

/*
 * Commands.
 */

#define MOXA_CMD_RESET_CARD 0x00
#define MOXA_CMD_RESET_CHAN 0x01
#define MOXA_CMD_OPEN 0x02
#define MOXA_CMD_CLOSE 0x03
#define MOXA_CMD_UNKNOWN1 0x04
#define MOXA_CMD_SET_MODE 0x05
#define MOXA_CMD_SET_BAUD 0x06
#define MOXA_CMD_SET_MODEM 0x07
#define MOXA_CMD_GET_MODEM 0x08
#define MOXA_CMD_TXINT_CONTROL 0x09
#define MOXA_CMD_CLEAR 0x0a
#define MOXA_CMD_BREAK_ON 0x0b
#define MOXA_CMD_BREAK_OFF 0x0c
#define MOXA_CMD_LOOPBACK_ON 0x0d
#define MOXA_CMD_LOOPBACK_OFF 0x0e
#define MOXA_CMD_INIT_CHAN 0x0f
#define MOXA_CMD_RESUME 0x10
#define MOXA_CMD_SNOOP_CTRL 0x11
#define MOXA_CMD_COUNT_CTRL 0x12
#define MOXA_CMD_BREAK_EVENT_ON 0x13
#define MOXA_CMD_MODEM_EVENT_ON 0x14
#define MOXA_CMD_SET_FLOW 0x15
#define MOXA_CMD_SEND_INT 0x16
#define MOXA_CMD_RAW_GET 0x17
#define MOXA_CMD_RAW_SET 0x18
#define MOXA_CMD_SET_XONOFF 0x19
#define MOXA_CMD_RAW_CMD 0x1a
#define MOXA_CMD_UNUSED1 0x1b
#define MOXA_CMD_UNUSED2 0x1c
#define MOXA_CMD_DISABLE_MODEM_EVENTS 0x1d

#define MOXA_MODE_LEN_5 0x00
#define MOXA_MODE_LEN_6 0x01
#define MOXA_MODE_LEN_7 0x02
#define MOXA_MODE_LEN_8 0x03
#define MOXA_MODE_STOP_1 0x00
#define MOXA_MODE_STOP_2 0x04
#define MOXA_MODE_PAR_NONE 0x00
#define MOXA_MODE_PAR_ODD 0x08
#define MOXA_MODE_PAR_EVEN 0x18
#define MOXA_MODE_PAR_MARK 0x28
#define MOXA_MODE_PAR_SPACE 0x38

#define MOXA_CLEAR_RX 0
#define MOXA_CLEAR_TX 1
#define MOXA_CLEAR_BOTH 2

#define MOXA_MAGIC_COOKIE 0x404
